/***************************************************
 *                A S T R O D O M E                *
 * gestione del tetto scorrevole dell'osservatorio *
 ***************************************************
 *
 * Copyright 2014-2015 by Ivaldo Cervini
 * 
 * 0.0.1.0 -  14 ottobre 2014 - Definizione del protocollo di comunicazione seriale
 * 0.1.0.0 - 23 novembre 2014 - Implementata la gestione dell'allarme Boltwood
 * 1.0.0.0 -  7 dicembre 2014 - Implementato nuovo protocollo seriale
 * 2.0.0.0 -    6 giugno 2015 - Riscritto per permettere ad ASCOM di conoscere lo stato del tetto
 * 2.1.0.0 -  14 ottobre 2017 - Modificato il protocollo di comunicazione con il driver ASCOM
 */
 
 /********************************
 * C O M A N D I   S E R I A L I *
 *********************************
 * O = Apertura del tetto
 * C = Chiusura del tetto
 * H = Interruzione del movimento del tetto
 * V = Versione del firmware
 * S = Richiesta dello status
 * 
 *****************************
 * Possibili stati del tetto *
 *****************************
 * 0 Aperto
 * 1 Chiuso
 * 2 In apertura
 * 3 In chiusura
 * 4 Errore
 */
 
// Connessioni di Arduino
int relay1 = 4; // Contatto di apertura del tetto
int relay2 = 5; // Contatto di chiusure del tetto
int relay3 = 6;
int relay4 = 7;
int endSwitch1 = 2; // Interruttore di fine corsa in apertura
int endSwitch2 = 3; // Interruttore di fine corsa in chiusura
int manualSwitch1 = 8; // Interruttore di apertura manuale
int manualSwitch2 = 9; // Interruttore di chiusura manuale
int meteoSwitch = 12; // Sensore di allarme meteo
int meteoLed = 13; // LED di allarme meteo
int powerSensor = A0; // Sensore della tensione di alimentazione a 5V

// Impostazioni iniziali
char versInfo[] = "2.1.0.0"; // Versione del firmware
long motorTimeOut = 1000; // Timeout del motore in centesimi di secondo
boolean meteoAlarm = HIGH; // Allarme meteo
byte roofStatus = 4; // Status del tetto

void setup() {
  // Operazioni di avvio del programma
  pinMode(relay1, OUTPUT);
  digitalWrite(relay1, HIGH);
  pinMode(relay2, OUTPUT);
  digitalWrite(relay2, HIGH);
  pinMode(relay3, OUTPUT);
  digitalWrite(relay3, HIGH);
  pinMode(relay4, OUTPUT);  
  digitalWrite(relay4, HIGH);
  pinMode(meteoLed, OUTPUT);
  digitalWrite(meteoLed, LOW);
  pinMode(endSwitch1, INPUT_PULLUP);
  pinMode(endSwitch2, INPUT_PULLUP);
  pinMode(manualSwitch1, INPUT_PULLUP);
  pinMode(manualSwitch2, INPUT_PULLUP);
  pinMode(meteoSwitch, INPUT_PULLUP);
  checkRoofStatus(); 
  Serial.begin(9600);
  delay(1000);
  }

void loop() {
  // Gestione dei comandi ricevuti dal PC
  delay(10); //
  switch(Serial.read()){
    case 79: // O - Richiesta di apertura del tetto
      roofOpen();
      break;
    case 67: // C - Richiesta di chiusura del tetto
      roofClose();
      break;
    case 83: // S - Richiesta dello status
      Serial.println(roofStatus);
      break;
    case 86: // V - Richiesta della versione
      Serial.println(versInfo);
      break;
  }
  
  // Verifica le richieste hardware
  if(digitalRead(manualSwitch1) == LOW) {roofOpen();} // Apertura manuale del tetto
  if(digitalRead(manualSwitch2) == LOW) {roofClose();} // Chiusura manuale del tetto
  if(analogRead(powerSensor) < 512){roofClose();} // Interruzione di corrente
  meteoAlarm = digitalRead(meteoSwitch); // Leggi lo stato del Boltwood
  digitalWrite(meteoLed, !meteoAlarm); // Accendi il LED in caso di allarme meteo
  if(meteoAlarm == LOW) {roofClose();} // Chiusura del tetto in caso di allarme meteo
}

void roofOpen(){
  // Apertura del tetto
  if(meteoAlarm == HIGH){ // Verifica che non sia attivo l'allarme meteo
    if(digitalRead(endSwitch1) == HIGH){ // Verifica che il tetto non sia già aperto
      roofStatus = 2;
      digitalWrite(relay1, LOW); // Attiva il contatto di apertura del tetto
      boolean roofMove = true; // Quando questa variabile diventa falsa il tetto si ferma
      long timer = 0; // Variabile usata come cronometro
      while(roofMove) { // Avvia un ciclo di verifica
        delay(10); // Pausa di un decimo di secondo
        timer++; // Incrementa il valore del cronometro
        if(timer > motorTimeOut){roofMove = false;} // È stato raggiunto il valore di timeout?
        if(digitalRead(endSwitch1) == LOW){roofMove = false;} // Il tetto è giunto a fine corsa?
        if(digitalRead(manualSwitch2) == LOW){roofMove = false;} // È stato premuto il pulsante di chiusura?
        switch(Serial.read()){ // Controlla se ci sono richieste dal PC
          case 83: // S - Richiesta dello status
            Serial.println(roofStatus);
            break;
          case 86: // V - Richiesta della versione
            Serial.println(versInfo);
            break;
          case 72: // H - Richiesta di arresto immediato del motore
            roofMove = false;
            break;
        }
      }
      digitalWrite(relay1, HIGH); // Disattiva il contatto di apertura del tetto
      while(digitalRead(manualSwitch2) == LOW){ // Attendi che il pulsante di chiusura venga rilasciato
        delay(10);
      }
      delay(500); // Pausa di mezzo secondo
      checkRoofStatus(); // Controlla la posizione del tetto
    }
  }
}

void roofClose(){
  // Chiusura del tetto
  if(digitalRead(endSwitch2) == HIGH){ // Verifica che il tetto non sia già chiuso
    roofStatus = 3;
    digitalWrite(relay2, LOW); // Attiva il contatto di chiusura del tetto
    boolean roofMove = true; // Quando questa variabile diventa falsa il tetto si ferma
    long timer = 0; // Variabile usata come cronometro
    while(roofMove) { // Avvia un ciclo di verifica
      delay(10); // Pausa di un decimo di secondo
      timer++; // Incrementa il valore del cronometro
      if(timer > motorTimeOut){roofMove = false;} // È stato raggiunto il valore di timeout?
      if(digitalRead(endSwitch2) == LOW){roofMove = false;} // Il tetto è giunto a fine corsa?
      if(digitalRead(manualSwitch1) == LOW){roofMove = false;} // È stato premuto il pulsante di apertura?
        switch(Serial.read()){ // Controlla se ci sono richieste dal PC
          case 83: // S - Richiesta dello status
            Serial.println(roofStatus);
            break;
          case 86: // V - Richiesta della versione
            Serial.println(versInfo);
            break;
          case 72: // H - Richiesta di arresto immediato del motore
            roofMove = false;
            break;
        }
    }
    digitalWrite(relay2, HIGH); // Disattiva il contatto di chiusura del tetto
    while(digitalRead(manualSwitch1) == LOW){ // Attendi che il pulsante di apertura venga rilasciato
      delay(10);
    }
    delay(500); // Pausa di mezzo secondo
    checkRoofStatus(); // Controlla la posizione del tetto
  }
}

void checkRoofStatus(){
  // Verifica lo status del tetto
  if(digitalRead(endSwitch1) == LOW && digitalRead(endSwitch2) == HIGH)
    {roofStatus = 0;} // Tetto aperto
  else if(digitalRead(endSwitch1) == HIGH && digitalRead(endSwitch2) == LOW)
    {roofStatus = 1;} // Tetto chiuso
  else
    {roofStatus = 4;} // Stato sconosciuto  
}
